#!/bin/bash
# ------------------------------------------------------------------------------
# (C) Copyright 1995 - 2022 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its
# affiliates and licensors (“Micro Focus”) are as may be set forth in the
# express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial
# errors or omissions contained herein. The information contained herein
# is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains
# confidential information and a valid license is required for possession,
# use or copying. If this work is provided to the U.S. Government,
# consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items
# are licensed to the U.S. Government under vendor's standard commercial
# license.
# ------------------------------------------------------------------------------
#
# script to setup postgres
# this script has to be executed as part of postgres post-install

Usage()
{
        echo ""
        echo "Usage: postgres-setup.sh [options]"
        echo "          -h                      Prints this summary"
        echo "          -p                      Postgres DB Port"
        echo ""

        exit 1;
}

if test $# -eq 0
then
        Usage
fi

#Get the parameters
while getopts p:h o
do
        case "$o" in
                     p)
			     DB_PORT="$OPTARG";;
                     h)
			     Usage;;
                     *) 
			     Usage;;
        esac
done

if [ ! -d ~postgres/data ]
then
    systemctl stop postgresql
    su - postgres -c "initdb -U postgres ~postgres/data"
    systemctl start postgresql
    systemctl enable postgresql
fi

#Modify the peer to trust in pg_hba.conf if host is local
grep -n "^local" ~postgres/data/pg_hba.conf | cut -f1 -d ":" | while read -r line ; do sed -i "$line s/peer/trust/" ~postgres/data/pg_hba.conf ; done

#Modify the ident to scram-sha-256
grep -n "^host" ~postgres/data/pg_hba.conf | cut -f1 -d ":" | while read -r line ; do sed -i "$line s/ident/scram-sha-256/" ~postgres/data/pg_hba.conf ; done

echo "host    all             all             0.0.0.0/0               scram-sha-256" >> ~postgres/data/pg_hba.conf

certFile=$(grep -w "^[[:blank:]]*ssl_cert_file" ~postgres/data/postgresql.conf | cut -d "=" -f2- | cut -d "'" -f 2)
sslCiphers=$(grep -w "^[[:blank:]]*ssl_ciphers" ~postgres/data/postgresql.conf | cut -d "=" -f2 | cut -d "'" -f 2)
listenAddr=$(grep -w "^[[:blank:]]*listen_addresses" ~postgres/data/postgresql.conf | cut -d "=" -f2 | cut -d "'" -f 2)
protocolVer=$(grep -w "^[[:blank:]]*ssl_min_protocol_version" ~postgres/data/postgresql.conf | cut -d "=" -f2 | cut -d "'" -f 2)
sslValue=$(grep -w "^[[:blank:]]*ssl" ~postgres/data/postgresql.conf | cut -d "=" -f2 | sed -e 's/^[ \t]*//')
portVal=$(grep -w "^[[:blank:]]*port" ~postgres/data/postgresql.conf | cut -d "=" -f2)

# Check for listen_addresses value
if test -z $listenAddr || test "$listenAddr" != "0.0.0.0"
then
    echo "listen_addresses = '0.0.0.0'" >> ~postgres/data/postgresql.conf
fi

# Check for port value
if test -z $portVal
then
    echo "port = $DB_PORT" >> ~postgres/data/postgresql.conf
fi

# Check for ssl_min_protocol_version value
if test -z $protocolVer || test "$protocolVer" != "TLSv1.2"
then
    echo "ssl_min_protocol_version = 'TLSv1.2'" >> ~postgres/data/postgresql.conf
fi

# If ssl entry is not present or not set to on then add/set ssl value to on
if test -z $sslValue  || test "$sslValue" != "on"
then
    echo "ssl = on" >> ~postgres/data/postgresql.conf
fi

# If ssl is on we need to provide the value for ssl_cert_file and ssl_key_file
# check ssl_cert_file is already present or not if not create and add the values

if [[ ${certFile::1} == "/" ]]
then
    certFilePath=$certFile
else
    certFilePath=~postgres/data/$certFile
fi

if test -z $certFile || test ! -s $certFilePath
then
    # generate self signed certificate
    TEMPDIR=`mktemp -d`
    /usr/share/sfcb/genSslCert.sh $TEMPDIR
    cp $TEMPDIR/server.pem ~postgres/data/servercert.pem
    cp $TEMPDIR/file.pem ~postgres/data/serverkey.pem

    chown postgres:postgres ~postgres/data/servercert.pem
    chown postgres:postgres ~postgres/data/serverkey.pem
    chmod 600 ~postgres/data/servercert.pem
    chmod 600 ~postgres/data/serverkey.pem

    rm -rf $TEMPDIR
    echo "ssl_cert_file = 'servercert.pem'" >> ~postgres/data/postgresql.conf
    echo "ssl_key_file = 'serverkey.pem'" >> ~postgres/data/postgresql.conf
fi

# Check sslCiphers are enabled and added if not then add it
if test -z $sslCiphers
then
    echo "ssl_ciphers = 'HIGH:MEDIUM:+3DES:!aNULL'" >> ~postgres/data/postgresql.conf
fi
systemctl restart postgresql.service
