#!/usr/bin/env python3
# ------------------------------------------------------------------------------
# Copyright 2023 Open Text.
#
# The only warranties for products and services of Open Text and its
# affiliates and licensors (“Open Text”) are as may be set forth in the
# express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional
# warranty. Open Text shall not be liable for technical or editorial
# errors or omissions contained herein. The information contained herein
# is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains
# confidential information and a valid license is required for possession,
# use or copying. If this work is provided to the U.S. Government,
# consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items
# are licensed to the U.S. Government under vendor's standard commercial
# license.
# ------------------------------------------------------------------------------
#
# script to modify apache vhost-ssl.conf for http to https redirection

res = []
vhost_file = "/etc/apache2/vhosts.d/vhost-ssl.conf"
opening_tag = "VirtualHost _default_:80"
ending_tag = "VirtualHost"
redirection_data = ['\n\tDocumentRoot  "/srv/www/htdocs"\n\n', '\tRewriteEngine On\n', '\tRewriteCond %{SERVER_PORT} !^443$\n', '\tRewriteRule ^/(umc|welcome)(/.*)?$ https://%{SERVER_NAME}/$1$2 [L,R]\n']

def open_file(filename, operation):
    try:
        file_open = open(filename, operation)
    except Exception as e:
        print("Exception occured while opening file : ", str(e))
        exit(2)
    return file_open

def close_file(file_name):
    file_name.close()

def read_conf_file(filename):
    global opening_tag_line
    global start_idx
    global end_idx

    file_open = open_file(vhost_file, "r")
    content_read = file_open.read()
    close_file(file_open)

    start_idx = content_read.find("<" + opening_tag  + ">")
    if start_idx != -1:
        opening_tag_line = content_read[:start_idx].count('\n') + 1

        end_idx = content_read.find("</" + ending_tag + ">", start_idx)
        res.append(content_read[start_idx+len(opening_tag)+2:end_idx])

def add_http_redirection():
    if start_idx != -1:
        if str(res).find("RewriteEngine On") == -1:
            file_open = open_file(vhost_file, "r+")
            lngth = 0
            try:
                #count the number of char until the desired line
                for i in range(1,opening_tag_line+1):
                    line = file_open.readline()
                    lngth = len(line) + lngth

                # read the rest of the file
                lines = file_open.read()
                #go back to insert position
                file_open.seek(lngth, 0)
                #insert sequence in file
                file_open.writelines(redirection_data)
                #write rest of file 
                file_open.write(lines)
                close_file(file_open)
            except Exception as e:
                print("Exception occured while reading and writing the file", str(e))
                exit(1)
    elif start_idx == -1:
        file_open = open_file(vhost_file, "a")
        try:
            file_open.write('\n<VirtualHost _default_:80>')
            file_open.writelines(redirection_data)
            file_open.write('</VirtualHost>')
            close_file(file_open)
        except Exception as e:
            print("Exception occured while reading and writing the file", str(e))
            exit(1)

def main():
    read_conf_file(vhost_file)
    add_http_redirection()

if __name__ == "__main__":
    main()
    exit(0)
